#!/usr/bin/env python3
import subprocess
import json
import logging
import logging.config
from threading import Timer
import re
import os
import hashlib
import zipfile
import requests
import time
import datetime
import platform
import shutil
import sys

def is_running_in_windows():
    os_type = platform.system()
    if os_type.lower() == 'windows':
        return True
    else:
        return False

if is_running_in_windows():
    import psutil

def kill_proc(proc, timeout):
    timeout["value"] = True
    proc.terminate()

#excecute system cmd
def executeCmd(cmd, isTrip=True, timeout_sec=3):
    logger = logging.getLogger("monitorLog")
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    timeout = {"value": False}
    timer = Timer(timeout_sec, kill_proc, [proc, timeout, cmd])
    timer.start()
    stdout, stderr = proc.communicate()
    timer.cancel()
    if isTrip:
        data = stdout.lstrip().rstrip()
    else:
        data = stdout
    result = {}
    result["data"] = data
    if len(stderr) != 0:
        logger.error("excute command %s failed! Failed reason: %s" % (cmd, stderr))
    else:
        logger.debug("excute command %s success!" % cmd)

    if proc.stdin:
        proc.stdin.close()
    if proc.stdout:
        proc.stdout.close()
    if proc.stderr:
        proc.stderr.close()
    return result

def getCloudProvider():
    if is_running_in_windows():
        return get_asset_tag()
    else:
        cmd = "cat /sys/class/dmi/id/chassis_asset_tag"
        str = executeCmd(cmd, isTrip=True)["data"]
        return str

def getNicByMac(mac):
    if is_running_in_windows():
        mac = mac.lower()
        for key in psutil.net_if_addrs().keys():
            value = psutil.net_if_addrs().get(key)
            macAddr = str(value[0]).split("'")[1]
            macAddr = macAddr.replace('-', ':').lower()
            if cmp(macAddr, mac) == 0:
                keyunicode = key.decode(sys.getfilesystemencoding())
                keyutf8 = keyunicode.encode("utf-8")
                return keyutf8
        return None
    else:
        cmd = "sudo /sbin/ifconfig  | grep %s |head -1 |awk -F' ' '{print $1}'" % (mac.upper())
        return executeCmd(cmd, isTrip=True)["data"]

def instanceLogger():
    loggerfilepath = os.path.abspath(os.path.join(__file__, "../", "logger.conf"))
    logging.config.fileConfig(loggerfilepath)
    global logger
    logger = logging.getLogger("monitorLog")

def instanceUpdateLogger():
    updateloggerfilepath = os.path.abspath(os.path.join(__file__, "../", "logger.conf"))
    logging.config.fileConfig(updateloggerfilepath)
    global updatelogger
    updatelogger = logging.getLogger("updateLog")

def getUpdateLogger():
    return updatelogger

def getLogger():
    return logger

def instanceOpenstackJson():
    global openstackJson
    logger = getLogger()
    try:
        url = getspecialConfigValue("OpenstackURL", name="globalConfig")
        result = requests.get(url).content
        #logger.info("instanceOpenstackJson:"+result)
        openstackJson = json.loads(result)
    except Exception,e:
        logger.error("instanceOpenstackJson error")
        logger.error(e)
        openstackJson = {}
    return openstackJson

def getOpenstackValue(key):
    result = "None"
    if openstackJson.has_key(key):
        result = openstackJson[key]
    return result

def getAKSKFromOpenStack():
    logger = getLogger()
    result = None
    try:
        url = getspecialConfigValue("OpenStackURL4AKSK", name="globalConfig")
        result = requests.get(url)
        if(result.status_code != 200):
            logger.error("Failed to get AK/SK from Metadata Service. " + url)
        result = result.content
    except Exception, e:
        logger.error("getAKSKFromOpenStack error")
        logger.error(e)
    return result

def instanceSpecialConfigJson(logger):
    global specialConfigJson
    try:
        path = os.path.join(os.path.dirname(__file__), 'config.json')
        fin = open(path, 'r')
        str = fin.read()
        specialConfigJson = json.loads(str)
    except Exception,e:
        logger.error("instanceSpecialConfigJson error")
        logger.error(e)
        specialConfigJson = {}
    finally:
        fin.close()

def getspecialConfigValue(key,name = "Bandwith"):
    result = "None"
    if specialConfigJson.has_key(name):
        if specialConfigJson[name].has_key(key):
            result = specialConfigJson[name][key]
    return result

def isExistThreadRun(selfId, scriptName):
    cmd = "ps aux | grep -w python | grep -F %s |grep -v grep | grep -v sudo |grep -v %s|head -1"%(scriptName, selfId)
    return executeCmd(cmd, isTrip=True)["data"]
    
def getCpuCoreNum():
    os_type = platform.system()

    if os_type.lower() == 'windows':
        return psutil.cpu_count(logical=False)
    else:
        cmd = "cat /proc/cpuinfo | grep 'processor' | wc -l"
        return executeCmd(cmd, isTrip=True)["data"]

def getMemoryConsumption():
    os_type = platform.system()
    if os_type.lower() == 'windows':
        return psutil.virtual_memory().percent
    else:
        logger = getLogger()
        cmd = "free -m| grep -E 'Mem' | awk -F' ' '{print $2}'"        
        total = int(executeCmd(cmd, isTrip=True)["data"])
        cmd = "free -m| grep -E 'Mem' | awk -F' ' '{print $3}'"
        used = int(executeCmd(cmd, isTrip=True)["data"])
        consumption = float(used)/float(total) * 100
        logger.info("consumption: %f" % consumption)
        return consumption

def getMemoryTotal():
    os_type = platform.system()

    if os_type.lower() == 'windows':
        return psutil.virtual_memory().total / 1024 / 1024
    else:
        cmd = "free -m| grep -E 'Mem' | awk -F' ' '{print $2}'"
        return executeCmd(cmd, isTrip=True)["data"]

def getDataProviderVersion(is_linux, logger):
    sha256_filename = ""
    version_file_name = ""
    if is_linux is True:
        sha256_filename = getspecialConfigValue("VersionFileLinuxSha256", name='globalConfig')
        version_file_name = getspecialConfigValue("VersionFileLinux", name='globalConfig')
    else:
        sha256_filename = getspecialConfigValue("VersionFileWindowsSha256", name='globalConfig')
        version_file_name = getspecialConfigValue("VersionFileWindows", name='globalConfig')

    bucket_path = getspecialConfigValue("BucketPath", name='globalConfig')
    obs_domain = getspecialConfigValue("ObsUrl", name='globalConfig')
    sha256_url = "%s/%s/%s" % (obs_domain, bucket_path, sha256_filename)
    version_file_url = "%s/%s/%s" % (obs_domain, bucket_path, version_file_name)

    version_file_local_file = os.path.join(os.path.dirname(__file__), version_file_name)

    sha256_file_exists = False

    response = requests.get(sha256_url)
    if response.status_code == 200:
        file_sha256_value = response.content.lstrip().rstrip()
        sha256_file_exists = True
    else:
        logger.debug("There is no sha256.")

    response = requests.get(version_file_url)
    if response.status_code == 200:
        version_content = response.content
        with open(version_file_local_file, "wb") as f:
            for chunk in response.iter_content(chunk_size=1024):
                f.write(chunk)
    else:
        raise Exception("Failed to get version file from %s. error code = %d" % (version_file_url, response.status_code))

    if sha256_file_exists:
        hash_value = sha256_checksum(version_file_local_file)
        if file_sha256_value != hash_value:
            raise Exception("Failed to verify sha256. sha256File is %s and calculated value is %s" % (
                file_sha256_value, hash_value))

    return float(json.loads(version_content).get('DataProviderVersion'))





def downloadDataProvider(is_linux, logger):
    sha256_filename = ""
    zip_file_name = ""
    bucket_path = getspecialConfigValue("BucketPath", name='globalConfig')
    obs_domain = getspecialConfigValue("ObsUrl", name='globalConfig')

    if is_linux is True:
        sha256_filename = getspecialConfigValue("DataProvdierFileLinuxSha256", name='globalConfig')
        zip_file_name = getspecialConfigValue("DataProviderFileLinux", name='globalConfig')
    else:
        sha256_filename = getspecialConfigValue("DataProvdierFileWindowsSha256", name='globalConfig')
        zip_file_name = getspecialConfigValue("DataProviderFileWindows", name='globalConfig')

    sha256_url = "%s/%s/%s" % (obs_domain, bucket_path, sha256_filename)
    zip_file_url = "%s/%s/%s" % (obs_domain, bucket_path, zip_file_name)

    sha256_exists = False
    response = requests.get(sha256_url)
    if response.status_code == 200:
        sha256_value = response.content.lstrip().rstrip()
        sha256_exists = True
    else:
        logger.debug("There is no sha256")

    zip_local_file = os.path.join(os.path.dirname(__file__), zip_file_name)
    if os.path.exists(zip_local_file):
        os.remove(zip_local_file)

    response = requests.get(zip_file_url, stream=True)
    if response.status_code == 200:
        with open(zip_local_file, "wb") as f:
            for chunk in response.iter_content(chunk_size=1024):
                f.write(chunk)
    else:
        raise Exception("Failed to get zip file from %s. error code = %s" % (zip_file_url, response.status_code))

    if sha256_exists is True:
        hash_value = sha256_checksum(zip_local_file)
        if sha256_value != hash_value:
            raise Exception("Sha256 is not equal,sha256File is %s and calculete is %s" % (
            sha256_value, hash_value))
    try:
        with zipfile.ZipFile(zip_local_file, 'r') as zip_object:
            for member in zip_object.namelist():
                filename = os.path.basename(member)
                #skip directories
                if not filename:
                    continue
                source = zip_object.open(member)
                target = file(os.path.join(os.path.dirname(__file__), filename), "wb")
                with source, target:
                    shutil.copyfileobj(source, target)
                    logger.debug("Extrating %s" % (filename))
            zip_object.close()

        if os.path.exists(zip_local_file):
            os.remove(zip_local_file)
    except zipfile.BadZipfile, e:
        logger.error("Bad zip file.")


def prepareDictData(inputDict):
    #virtualization_solution ----to upper
    #memory_over_provisioning ---- 0:no 1:yes
    logger = getLogger()
    try:
        if(inputDict.get('virtualization_solution')):
            inputDict['virtualization_solution'] = inputDict['virtualization_solution'].upper()
        if (inputDict.get('memory_over_provisioning') and int(1 == inputDict.get('memory_over_provisioning'))):
            inputDict['memory_over_provisioning'] = 'yes'
        else:
            inputDict['memory_over_provisioning'] = 'no'
    except Exception,e:
        logger.error('prepareDictData failed')
        logger.error(e)

def isValidMac(mac):
    if re.match(r"^\s*([0-9a-fA-F]{2}:){5,5}[0-9a-fA-F]{2}\s*$", mac):
        return True
    return False

def sha256_checksum(filename, block_size=65536):
    sha256 = hashlib.sha256()
    with open(filename, 'rb') as f:
        for block in iter(lambda: f.read(block_size), b''):
            sha256.update(block)
    return sha256.hexdigest()
	
def isNeedFreshAKSK(akskStr):
    isNeed = False
    if len(akskStr) == 0:
        isNeed = True
    else:
        shareAKSKJson = json.loads(akskStr)
        expiredTime = shareAKSKJson.get("expires_at")
        expiredTimeFormat = datetime.datetime.strptime(expiredTime,"%Y-%m-%dT%H:%M:%S.%fZ")
        curTimeFormat = datetime.datetime.utcnow()
        # if aksk will expired in 5 minutes, need to reget aksk
        if curTimeFormat + datetime.timedelta(minutes=5) >= expiredTimeFormat:
            isNeed = True
    return isNeed

def isOpenStackAKSKJsonValid(aksk):
    """Check aksk json format.
    {
        "expires_at": "2015-02-27T18:30:59.999999Z",
        "access": "xxxXXXxxxxxx",
        "secret": "xxxXXXXXXXXXXXX",
        "securitytoken": "xxxXXXXXXXXXXXX"
        }
    """
    legalAKSKJsonKey = ["expires_at", "access", "secret", "securitytoken"]
    jsonKeys = aksk.keys()
    for key in legalAKSKJsonKey:
        if key not in jsonKeys:
            return False
    jsonValues = aksk.values()
    for value in jsonValues:
        if len(value) == 0:
            return False
    return True

def getAKSK():
    logger.debug("Try to get AK/SK from Metatdata Service.")
    tryTimes = 3
    tmpJsonString = None
    while tryTimes > 0:
        tmpJsonString = getAKSKFromOpenStack()
        if tmpJsonString is not None:
            break
        else:
            time.sleep(10)
            tryTimes -= 1
    if tmpJsonString is None:
        logger.error('Failed to get AKSK from OpenStack')
        return None
    try:
        akskJsonDic = json.loads(tmpJsonString)["credential"]
    except Exception, e:
        logger.error('Failed to loads String to Json')
        logger.error(e)
        return None

    if isOpenStackAKSKJsonValid(akskJsonDic) is True:
        logger.debug('Succeeded to get AKSK from OpenStack')
        return akskJsonDic
    else:
        logger.error('AKSK Json reponse is invalid')
        return None

def get_asset_tag():
    from win32com.client import GetObject
    objWMI = GetObject('winmgmts:').Instancesof('Win32_SystemEnclosure')
    for obj in objWMI:
        if obj.SMBIOSAssetTag != None:
            asset_tag = obj.SMBIOSAssetTag
            return asset_tag

def get_device_name(wwn, volume_identify):
    if(wwn.startswith('scsi-3')):
        if(is_running_in_windows()):
            return volume_identify
        else:
            try:
                device_name = os.path.basename(os.readlink("/dev/disk/by-id/%s" % wwn))
                return device_name
            except Exception, e:
                logger.warn('/dev/disk/by-id/%s is not a symbolic link.' % wwn)
                logger.warn(e)
                return volume_identify
    else:
        return volume_identify